# -*- coding: utf-8 -*-
"""

Get some chips

"""

from sklearn.metrics import r2_score as R2
from sklearn.metrics import mean_squared_error as MSE
from sklearn.metrics import explained_variance_score as EVS
from sklearn.metrics import mean_absolute_error as MAE

from lightgbm import LGBMRegressor
import pandas as pd
from matplotlib import pyplot as plt
import numpy as np
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
import optuna
import warnings
warnings.filterwarnings("ignore", category=FutureWarning)

plt.rc('font', family='Times New Roman', size=30)

trainPath = r'训练集-数据增强.xlsx'
testPath = r'测试集.xlsx'

trainBase = pd.read_excel(trainPath)
testBase = pd.read_excel(testPath)

XName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P']
YName = 'Ea'

xTrain_ = trainBase[XName].values
yTrain_ = trainBase[YName].values

xTest = testBase[XName].values
yTest = testBase[YName].values
#%%
KF = KFold(n_splits=5, random_state=2023, shuffle=True)
def objectFun(trial):
    #设置超参数范围

    n_estimators = trial.suggest_int('n_estimators', 10, 1000)
    max_depth = trial.suggest_int('max_depth', 2, 16)
    learning_rate = trial.suggest_float('learning_rate', 0, 1)
    random_state = trial.suggest_int('random_state', 0, 3000)

    LGBM = LGBMRegressor(n_estimators=n_estimators,
                         max_depth=max_depth,
                         learning_rate=learning_rate,
                         random_state=random_state)
    #返回五折交叉验证R2用作判断标准
    scores = []
    for trainIndex, valIndex in KF.split(xTrain_):
        xTrain, xVal = xTrain_[trainIndex], xTrain_[valIndex]
        yTrain, yVal = yTrain_[trainIndex], yTrain_[valIndex]
        LGBM.fit(xTrain, yTrain)
        LGBMPre = LGBM.predict(xVal)
        flodScore = R2(yVal, LGBMPre)
        scores.append(flodScore)
    score = sum(scores) / len(scores)
    

    return LGBMR2

    
#实例化搜寻函数
study = optuna.create_study(direction='maximize')
#设置超参数搜寻次数
nTrials=2000
#开始优化
study.optimize(objectFun, n_trials=nTrials)
bestParam_ = study.best_trial.params
print('最佳参数:', study.best_trial.params)

#%%
bestParam = {'n_estimators': 369,
             'max_depth': 2,
             'learning_rate': 0.09948690128133542,
             'random_state': 739}
LGBM = LGBMRegressor(n_estimators=bestParam['n_estimators'],
                     max_depth=bestParam['max_depth'],
                     learning_rate=bestParam['learning_rate'],
                     random_state=bestParam['random_state'])

#拟合模型
LGBM.fit(xTrain_, yTrain_)
#调用拟合好的模型进行预测
LGBMPre = LGBM.predict(xTest)
#计算指标
LGBMMAE = MAE(yTest, LGBMPre)
LGBMMSE = MSE(yTest, LGBMPre)
LGBMRMSE = np.sqrt(MSE(yTest, LGBMPre))
LGBMEVS = EVS(yTest, LGBMPre)
LGBMR2 = R2(yTest, LGBMPre)

print('\nLightGBM 评价指标:\n')
print('MAE', LGBMMAE)
print('MSE', LGBMMSE)
print('RMSE', LGBMRMSE)
print('EVS', LGBMEVS)
print('R2', LGBMR2)

plt.figure(figsize=(10, 8), dpi=100)
plt.scatter(yTest, LGBMPre, color='black')
plt.plot([-5, 5], [-5, 5], linestyle='dashed', color='royalblue')
plt.xlim(-5, 5)
plt.ylim(-5, 5)
plt.xlabel('True')
plt.ylabel('Prediction')
#%%
plt.figure(figsize=(10,8),dpi=100)
nameScore = dict(zip(XName, LGBM.feature_importances_ / sum(LGBM.feature_importances_)))
nameScore = dict(sorted(nameScore.items(),key = lambda x:x[1],reverse = False))
plt.barh(list(nameScore.keys()),list(nameScore.values()), color='black')
plt.title('LightGBM Importance')
plt.xlabel('Score')
plt.ylabel('Features')
print('特征重要性分数')
print(nameScore)

import pickle
with open(r'模型保存/LightGBM.model', 'wb') as File:
    pickle.dump(LGBM, File)