# -*- coding: utf-8 -*-
"""

Get some chips

"""

from sklearn.metrics import r2_score as R2
from sklearn.metrics import mean_squared_error as MSE
from sklearn.metrics import explained_variance_score as EVS
from sklearn.metrics import mean_absolute_error as MAE
from xgboost import XGBRegressor
import pandas as pd
from matplotlib import pyplot as plt
import numpy as np
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
import optuna
import warnings
warnings.filterwarnings("ignore", category=FutureWarning)

plt.rc('font', family='Times New Roman', size=20)

trainPath = r'训练集-数据增强.xlsx'
testPath = r'测试集.xlsx'

trainBase = pd.read_excel(trainPath)
testBase = pd.read_excel(testPath)

XName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P']
YName = 'Ea'

xTrain_ = trainBase[XName].values
yTrain_ = trainBase[YName].values

xTest = testBase[XName].values
yTest = testBase[YName].values

#%%
KF = KFold(n_splits=5, random_state=2023, shuffle=True)
def objectFun(trial):
    #设置超参数范围
    max_depth = trial.suggest_int('max_depth', 1, 20)
    n_estimators = trial.suggest_int('n_estimators', 10, 1000)
    learning_rate = trial.suggest_float('learning_rate', 0, 1)
    reg_alpha = trial.suggest_float('reg_alpha', 0, 50)
    reg_lambda = trial.suggest_float('reg_lambda', 0, 50)
    gamma = trial.suggest_float('gamma', 0, 50)
    random_state = trial.suggest_int('random_state', 0, 3000)
    
    XGB = XGBRegressor(max_depth=max_depth,
                       n_estimators=n_estimators,
                       learning_rate=learning_rate,
                       reg_alpha=reg_alpha,
                       reg_lambda=reg_lambda,
                       gamma=gamma,
                       random_state=random_state)
    #返回五折交叉验证R2用作判断标准
    scores = []
    for trainIndex, valIndex in KF.split(xTrain_):
        xTrain, xVal = xTrain_[trainIndex], xTrain_[valIndex]
        yTrain, yVal = yTrain_[trainIndex], yTrain_[valIndex]
        XGB.fit(xTrain, yTrain)
        XGBPre = XGB.predict(xVal)
        flodScore = R2(yVal, XGBPre)
        scores.append(flodScore)
    score = sum(scores) / len(scores)
    
    return score

    
#实例化搜寻函数
study = optuna.create_study(direction='maximize')
#设置超参数搜寻次数
nTrials=200
#开始优化
study.optimize(objectFun, n_trials=nTrials)
bestParam_ = study.best_trial.params
print('最佳参数:', study.best_trial.params)



#%%
bestParam = {'max_depth': 16,
             'n_estimators': 55,
             'learning_rate': 0.46142791963905755,
             'reg_alpha': 3.724143861264925,
             'reg_lambda': 12.591553701939448,
             'gamma': 0.011691417422209353,
             'random_state': 974}
XGB = XGBRegressor(max_depth=bestParam['max_depth'],
                   n_estimators=bestParam['n_estimators'],
                   learning_rate=bestParam['learning_rate'],
                   reg_alpha=bestParam['reg_alpha'],
                   reg_lambda=bestParam['reg_lambda'],
                   gamma=bestParam['gamma'],
                   random_state=bestParam['random_state'])

#拟合模型
XGB.fit(xTrain_, yTrain_)
#调用拟合好的模型进行预测
XGBPre = XGB.predict(xTest)
#计算指标
XGBMAE = MAE(yTest, XGBPre)
XGBMSE = MSE(yTest, XGBPre)
XGBRMSE = np.sqrt(MSE(yTest, XGBPre))
XGBEVS = EVS(yTest, XGBPre)
XGBR2 = R2(yTest, XGBPre)

print('\nXGBoost 评价指标:\n')
print('MAE', XGBMAE)
print('MSE', XGBMSE)
print('RMSE', XGBRMSE)
print('EVS', XGBEVS)
print('R2', XGBR2)

plt.figure(figsize=(10, 8), dpi=100)
plt.scatter(yTest, XGBPre, color='black')
plt.plot([-5, 5], [-5, 5], linestyle='dashed', color='royalblue')
plt.xlim(-5, 5)
plt.ylim(-5, 5)
plt.xlabel('True')
plt.ylabel('Prediction')

import pickle
with open(r'模型保存/XGboost.model', 'wb') as File:
    pickle.dump(XGB, File)