# -*- coding: utf-8 -*-
"""

Get some chips

"""

from sklearn.metrics import r2_score as R2
from sklearn.metrics import mean_squared_error as MSE
from sklearn.metrics import explained_variance_score as EVS
from sklearn.metrics import mean_absolute_error as MAE

from catboost import CatBoostRegressor
import pandas as pd
from matplotlib import pyplot as plt
import numpy as np
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
import optuna
import warnings
warnings.filterwarnings("ignore", category=FutureWarning)

plt.rc('font', family='Times New Roman', size=20)

trainPath = r'训练集-数据增强.xlsx'
testPath = r'测试集.xlsx'

trainBase = pd.read_excel(trainPath)
testBase = pd.read_excel(testPath)

XName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P']
YName = 'Ea'

xTrain_ = trainBase[XName].values
yTrain_ = trainBase[YName].values

xTest = testBase[XName].values
yTest = testBase[YName].values

#%%
KF = KFold(n_splits=5, random_state=2023, shuffle=True)
def objectFun(trial):
    #设置超参数范围
    n_estimators = trial.suggest_int('n_estimators', 10, 1000)
    max_depth = trial.suggest_int('max_depth', 1, 16)
    learning_rate = trial.suggest_float('learning_rate', 0, 1)
    random_state = trial.suggest_int('random_state', 0, 3000)
    
    CAT = CatBoostRegressor(n_estimators=n_estimators,
                            max_depth=max_depth,
                            learning_rate=learning_rate,
                            verbose=False,
                            random_state=random_state)
    #返回五折交叉验证R2用作判断标准
    scores = []
    for trainIndex, valIndex in KF.split(xTrain_):
        xTrain, xVal = xTrain_[trainIndex], xTrain_[valIndex]
        yTrain, yVal = yTrain_[trainIndex], yTrain_[valIndex]
        CAT.fit(xTrain, yTrain)
        CATPre = CAT.predict(xVal)
        flodScore = R2(yVal, CATPre)
        scores.append(flodScore)
    score = sum(scores) / len(scores)
    return score

    
#实例化搜寻函数
study = optuna.create_study(direction='maximize')
#设置超参数搜寻次数
nTrials=200
#开始优化
study.optimize(objectFun, n_trials=nTrials)
bestParam_ = study.best_trial.params
print('最佳参数:', study.best_trial.params)



#%%
bestParam = {'n_estimators': 99,
             'max_depth': 3,
             'learning_rate': 0.07806672469294972,
             'random_state': 2404}

CAT = CatBoostRegressor(max_depth=bestParam['max_depth'],
                        n_estimators=bestParam['n_estimators'],
                        learning_rate=bestParam['learning_rate'],
                        verbose=False,
                        random_state=bestParam['random_state'])

#拟合模型
CAT.fit(xTrain_, yTrain_)
#调用拟合好的模型进行预测
CATPre = CAT.predict(xTest)
#计算指标
CATMAE = MAE(yTest, CATPre)
CATMSE = MSE(yTest, CATPre)
CATRMSE = np.sqrt(MSE(yTest, CATPre))
CATEVS = EVS(yTest, CATPre)
CATR2 = R2(yTest, CATPre)

print('\nCatBoost 评价指标:\n')
print('MAE', CATMAE)
print('MSE', CATMSE)
print('RMSE', CATRMSE)
print('EVS', CATEVS)
print('R2', CATR2)

plt.figure(figsize=(10, 8), dpi=100)
plt.scatter(yTest, CATPre, color='black')
plt.plot([-5, 5], [-5, 5], linestyle='dashed', color='royalblue')
plt.xlim(-5, 5)
plt.ylim(-5, 5)
plt.xlabel('True')
plt.ylabel('Prediction')

plt.figure(figsize=(10,8),dpi=100)
nameScore = dict(zip(XName, CAT.feature_importances_ / sum(CAT.feature_importances_)))
nameScore = dict(sorted(nameScore.items(),key = lambda x:x[1],reverse = False))
plt.barh(list(nameScore.keys()),list(nameScore.values()), color='black')
plt.title('CatBoost Importance')
plt.xlabel('Score')
plt.ylabel('Features')
print('特征重要性分数')
print(nameScore)
#%%
import pickle
with open(r'模型保存/CatBoost.model', 'wb') as File:
    pickle.dump(CAT, File)