# -*- coding: utf-8 -*-
"""

Get some chips

"""

from sklearn.metrics import r2_score as R2
from sklearn.metrics import mean_squared_error as MSE
from sklearn.metrics import explained_variance_score as EVS
from sklearn.metrics import mean_absolute_error as MAE
import tensorflow as tf
import pandas as pd
from matplotlib import pyplot as plt
import numpy as np
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import KFold
from sklearn.model_selection import train_test_split
import optuna
import warnings
from tensorflow.keras.models import Sequential
from tensorflow.keras.layers import Conv1D, MaxPooling1D, Flatten, Dense, Input
import os
import random
seed = 2023
os.environ['PYTHONHASHSEED'] = str(seed) 
random.seed(seed)
np.random.seed(seed)
tf.random.set_seed(seed)
os.environ['TF_DETERMINISTIC_OPS'] = '1'
os.environ['TF_CUDNN_DETERMINISTIC'] = '1'

warnings.filterwarnings("ignore", category=FutureWarning)

plt.rc('font', family='Times New Roman', size=20)

trainPath = r'训练集-数据增强.xlsx'
testPath = r'测试集.xlsx'

trainBase = pd.read_excel(trainPath)
testBase = pd.read_excel(testPath)

XName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P']
YName = 'Ea'

xTrain_ = trainBase[XName].values
yTrain_ = trainBase[YName].values

xTest = testBase[XName].values
yTest = testBase[YName].values

def DNNModel(activation, DenseNum):
    DNN = Sequential()
    DNN.add(Dense(512, activation=activation))
    DNN.add(Dense(256, activation=activation))
    DNN.add(Dense(128, activation=activation))
    for I in range(DenseNum-3):
        DNN.add(Dense(32, activation=activation))
    DNN.add(Dense(1))
    
    DNN.compile(loss='mse',
                optimizer=tf.keras.optimizers.Adam(learning_rate=0.001),
                metrics=['mae'])
    
    return DNN



#%%
KF = KFold(n_splits=5, random_state=2023, shuffle=True)
def objectFun(trial):
    #设置超参数范围
    activation = trial.suggest_categorical('activation', ['elu', 'relu', 'gelu'])
    DenseNum = trial.suggest_int('DenseNum', 2,6)
    model = DNNModel(activation, DenseNum)
    
    #返回五折交叉验证R2用作判断标准
    scores = []
    for trainIndex, valIndex in KF.split(xTrain_):
        xTrain, xVal = xTrain_[trainIndex], xTrain_[valIndex]
        yTrain, yVal = yTrain_[trainIndex], yTrain_[valIndex]
        model.fit(xTrain, yTrain)
        modelPre = model.predict(xVal)
        flodScore = R2(yVal, modelPre)
        scores.append(flodScore)
    score = sum(scores) / len(scores)
    

    return score

    
#实例化搜寻函数
study = optuna.create_study(direction='maximize')
#设置超参数搜寻次数
nTrials=200
#开始优化
study.optimize(objectFun, n_trials=nTrials)
bestParam_ = study.best_trial.params
print('最佳参数:', study.best_trial.params)



#%%
bestParam = {'activation': 'elu',
             'DenseNum': 5}


model = DNNModel(activation=bestParam['activation'],
                 DenseNum=bestParam['DenseNum'])

#拟合模型
model.fit(xTrain_, yTrain_, epochs=100, batch_size=8, verbose=1)
#调用拟合好的模型进行预测
modelPre = model.predict(xTest)
#计算指标
modelMAE = MAE(yTest, modelPre)
modelMSE = MSE(yTest, modelPre)
modelRMSE = np.sqrt(MSE(yTest, modelPre))
modelEVS = EVS(yTest, modelPre)
modelR2 = R2(yTest, modelPre)

print('\nDNN 评价指标:\n')
print('MAE', modelMAE)
print('MSE', modelMSE)
print('RMSE', modelRMSE)
print('EVS', modelEVS)
print('R2', modelR2)

plt.figure(figsize=(10, 8), dpi=100)
plt.scatter(yTest, modelPre, color='black')
plt.plot([-5, 5], [-5, 5], linestyle='dashed', color='royalblue')
plt.xlim(-5, 5)
plt.ylim(-5, 5)
plt.xlabel('True')
plt.ylabel('Prediction')

model.save(r'模型保存\DNN.h5')
