# -*- coding: utf-8 -*-
"""

Get some chips

"""

import pickle
import tensorflow as tf
import pandas as pd
from sklearn.metrics import r2_score as R2
from sklearn.metrics import mean_squared_error as MSE
from sklearn.metrics import explained_variance_score as EVS
from sklearn.metrics import mean_absolute_error as MAE
import numpy as np
import seaborn as sns
from matplotlib import pyplot as plt

RFFilePath = r'模型保存\随机森林.model'
CATFilePath = r'模型保存\CatBoost.model'
LGBMFilePath = r'模型保存\LightGBM.model'
XGBFilePath = r'模型保存\XGboost.model'

#原本的训练集
allPath = r'训练集-151个点.xlsx'
predPath = r'预测集.xls'


allBase = pd.read_excel(allPath)
predBase = pd.read_excel(predPath)
#对训练集
columnMeans = predBase.mean()
# 用均值填充 NaN 值
predBase = predBase.fillna(columnMeans)

with open(RFFilePath, 'rb') as File:
    RF = pickle.load(File)
    

with open(CATFilePath, 'rb') as File:
    CAT = pickle.load(File)
    

with open(LGBMFilePath, 'rb') as File:
    LGBM = pickle.load(File)
    

with open(XGBFilePath, 'rb') as File:
    XGB = pickle.load(File)
    
CNN = tf.keras.models.load_model('模型保存\CNN.h5')
DNN = tf.keras.models.load_model('模型保存\DNN.h5')

XName = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P']
predBase.columns =  ['ID', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P']
#%%




allBaseRF = allBase.copy()
allBaseRF['Ea-Predicted'] = RF.predict(allBase[XName])

allBaseCAT = allBase.copy()
allBaseCAT['Ea-Predicted'] = CAT.predict(allBase[XName])

allBaseLGBM = allBase.copy()
allBaseLGBM['Ea-Predicted'] = LGBM.predict(allBase[XName])

allBaseXGB = allBase.copy()
allBaseXGB['Ea-Predicted'] = XGB.predict(allBase[XName])

allBaseCNN = allBase.copy()
allBaseCNN['Ea-Predicted'] = CNN.predict(allBase[XName])

allBaseDNN = allBase.copy()
allBaseDNN['Ea-Predicted'] = DNN.predict(allBase[XName])

allBaseRF.to_excel(r'训练集-151个点-结果\随机森林结果-训练集.xlsx')
allBaseCAT.to_excel(r'训练集-151个点-结果\CatBoost结果-训练集.xlsx')
allBaseLGBM.to_excel(r'训练集-151个点-结果\LightGBM结果-训练集.xlsx')
allBaseCNN.to_excel(r'训练集-151个点-结果\CNN结果-训练集.xlsx')
allBaseDNN.to_excel(r'训练集-151个点-结果\DNN结果-训练集.xlsx')
#%%
#对预测集
preBaseRF = predBase.copy()
preBaseRF['Ea-Predicted'] = RF.predict(predBase.drop(['ID'], axis=1))

preBaseCAT = predBase.copy()
preBaseCAT['Ea-Predicted'] = CAT.predict(predBase.drop(['ID'], axis=1))

preBaseLGBM = predBase.copy()
preBaseLGBM['Ea-Predicted'] = LGBM.predict(predBase.drop(['ID'], axis=1))

preBaseXGB = predBase.copy()
preBaseXGB['Ea-Predicted'] = XGB.predict(predBase.drop(['ID'], axis=1))

preBaseCNN = predBase.copy()
preBaseCNN['Ea-Predicted'] = CNN.predict(predBase.drop(['ID'], axis=1))

preBaseDNN = predBase.copy()
preBaseDNN['Ea-Predicted'] = DNN.predict(predBase.drop(['ID'], axis=1))

preBaseRF.to_excel(r'预测集结果\随机森林结果-预测集.xlsx')
preBaseCAT.to_excel(r'预测集结果\CatBoost结果-预测集.xlsx')
preBaseLGBM.to_excel(r'预测集结果\LightGBM结果-预测集.xlsx')
preBaseXGB.to_excel(r'预测集结果\XGboost结果-预测集.xlsx')
preBaseCNN.to_excel(r'预测集结果\CNN结果-预测集.xlsx')
preBaseDNN.to_excel(r'预测集结果\DNN结果-预测集.xlsx')
#%%

def countMetrics(trueValue, PredValue):
    modelMAE = MAE(trueValue, PredValue)
    modelMSE = MSE(trueValue, PredValue)
    modelRMSE = np.sqrt(MSE(trueValue, PredValue))
    modelEVS = EVS(trueValue, PredValue)
    modelR2 = R2(trueValue, PredValue)
    return modelMAE, modelMSE, modelRMSE, modelEVS, modelR2

RFMetirc = countMetrics(allBaseRF['Ea'], allBaseRF['Ea-Predicted'])
CATMetirc = countMetrics(allBaseCAT['Ea'], allBaseCAT['Ea-Predicted'])
LGBMMetirc = countMetrics(allBaseLGBM['Ea'], allBaseLGBM['Ea-Predicted'])
XGBMetirc = countMetrics(allBaseXGB['Ea'], allBaseXGB['Ea-Predicted'])
CNNMetirc = countMetrics(allBaseCNN['Ea'], allBaseCNN['Ea-Predicted'])
DNNMetirc = countMetrics(allBaseDNN['Ea'], allBaseDNN['Ea-Predicted'])

metricsBase = pd.DataFrame(columns=['均方误差MSE', '平均绝对误差MAE', '均方根误差RMSE', '可解释性方差EVS', '决定系数R2'])
metricsBase.loc['随机森林', :] = RFMetirc
metricsBase.loc['CatBoost', :] = CATMetirc
metricsBase.loc['LightGBM', :] = LGBMMetirc
metricsBase.loc['XGBoost', :] = XGBMetirc
metricsBase.loc['CNN', :] = CNNMetirc
metricsBase.loc['DNN', :] = DNNMetirc
metricsBase.to_excel('精度指标.xlsx')
#%%
LGBMScore = {'K': 0.020518358531317494, 'O': 0.020518358531317494, 'M': 0.026997840172786176, 'N': 0.03131749460043196, 'A': 0.04103671706263499, 'C': 0.04967602591792657, 'L': 0.056155507559395246, 'G': 0.05723542116630669, 'D': 0.06479481641468683, 'P': 0.07127429805615551, 'J': 0.07775377969762419, 'I': 0.08315334773218143, 'F': 0.08747300215982722, 'H': 0.10151187904967603, 'B': 0.10367170626349892, 'E': 0.10691144708423327}
CATScore = {'C': 0.007076763394361346, 'H': 0.013113658960626263, 'O': 0.013372510353700049, 'N': 0.01363220316557116, 'E': 0.013698767498629577, 'M': 0.027094086517769976, 'L': 0.028013284822302368, 'A': 0.03594080184997477, 'D': 0.03741232074573031, 'J': 0.038917383132556466, 'G': 0.04616014018267785, 'K': 0.05223688334455506, 'I': 0.06396210571032342, 'P': 0.07963465262224192, 'F': 0.09894707645555346, 'B': 0.430787361243426}
ScoreBase = pd.DataFrame(columns=XName)
for I in LGBMScore:
    ScoreBase.loc['LightGBM', I] = LGBMScore[I]
for I in CATScore:
    ScoreBase.loc['CatBoost', I] = CATScore[I]
ScoreBase.to_excel(r'特征重要性数据\特征重要性分数.xlsx')
#%%
RFCorr = preBaseRF.drop(['ID'], axis=1).corr()
CATCorr = preBaseCAT.drop(['ID'], axis=1).corr()
LGBMCorr = preBaseLGBM.drop(['ID'], axis=1).corr()
XGBCorr = preBaseXGB.drop(['ID'], axis=1).corr()
CNNCorr = preBaseCNN.drop(['ID'], axis=1).corr()
DNNCorr = preBaseDNN.drop(['ID'], axis=1).corr()
RFCorr.to_excel(r'皮尔逊相关性数据\随机森林皮尔逊相关性数据.xlsx')
CATCorr.to_excel(r'皮尔逊相关性数据\CatBoost皮尔逊相关性数据.xlsx')
LGBMCorr.to_excel(r'皮尔逊相关性数据\LightGBM皮尔逊相关性数据.xlsx')
XGBCorr.to_excel(r'皮尔逊相关性数据\XGboost皮尔逊相关性数据.xlsx')
CNNCorr.to_excel(r'皮尔逊相关性数据\CNN皮尔逊相关性数据.xlsx')
DNNCorr.to_excel(r'皮尔逊相关性数据\DNN皮尔逊相关性数据.xlsx')
#%%
plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(RFCorr, annot=True, fmt='.2f')
plt.title('Random Forest Pearson')
plt.show()

plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(CATCorr, annot=True, fmt='.2f')
plt.title('CatBoost Pearson')
plt.show()

plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(LGBMCorr, annot=True, fmt='.2f')
plt.title('LightGBM Pearson')
plt.show()

plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(XGBCorr, annot=True, fmt='.2f')
plt.title('XGBoost Pearson')
plt.show()

plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(CNNCorr, annot=True, fmt='.2f')
plt.title('CNN Pearson')
plt.show()

plt.figure(figsize=(20, 16), dpi=100)
sns.heatmap(DNNCorr, annot=True, fmt='.2f')
plt.title('DNN Pearson')
plt.show()